/********************************************************************************
Tampa IASA (Tampa, FL. 2011-02-24)

Dmitri V. Korotkevitch "Database Design for non-database architects"

( dmitri@aboutsqlserver.com )

INIT DATABASE SCRIPT
********************************************************************************/

use IASA
go

set nocount on
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'LargeRow'
)
	drop table dbo.LargeRow
go

create table dbo.LargeRow
(
	ID int not null,
	IntField int not null, 
	CharField char(2000) not null,
	
	constraint PK_LargeRow
	primary key clustered(Id)
)
go


if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'SmallRow'
)
	drop table dbo.SmallRow
go

create table dbo.SmallRow
(
	ID int not null,
	IntField int not null, 
	CharField varchar(2000) not null,
	
	constraint PK_SmallRow
	primary key clustered(Id)
)
go

begin tran
	;with CTE(Num)
	as
	(
		select 1
		union all
		select Num + 1
		from CTE
		where Num < 10000
	)
	insert into dbo.LargeRow(ID, IntField, CharField)
		select Num, Num, 'Placeholder'
		from CTE
	option (maxrecursion 0)
	
	;with CTE(Num)
	as
	(
		select 1
		union all
		select Num + 1
		from CTE
		where Num < 10000
	)
	insert into dbo.SmallRow(ID, IntField, CharField)
		select Num, Num, 'Placeholder'
		from CTE
	option (maxrecursion 0)
commit
go

