/******************************************************
Tampa IASA (Tampa, FL. 2011-02-24)

Dmitri V. Korotkevitch "Database Design for non-database architects"

( dmitri@aboutsqlserver.com )

Design pattern to illuminate blobs or seldom used fields 

Good when:
Table has a lot of data
Blobs are rarely accessed
System has a lot of queries which scan large # of rows in the table

Tables created and populated in "Init Db.sql" script

Enable "Include Actual Execution Plan" option
********************************************************/

use IASA
go

set nocount on
go


if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'MainData'
)
	drop table dbo.MainData
go

create table dbo.MainData
(
	ID int not null,
	IntField int not null, 
	
	constraint PK_MainData
	primary key clustered(Id)
)
go


if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'BlobData'
)
	drop table dbo.BlobData
go

create table dbo.BlobData
(
	ID int not null,
	CharField char(2000) not null,
	
	constraint PK_BlobData
	primary key clustered(Id),

	constraint FK_BlobData_MainData
	foreign key(ID) references dbo.MainData(ID)
	on delete cascade
)
go

begin tran
	;with CTE(Num)
	as
	(
		select 1
		union all
		select Num + 1
		from CTE
		where Num < 10000
	)
	insert into dbo.MainData(ID, IntField)
		select Num, Num
		from CTE
	option (maxrecursion 0)

	;with CTE(Num)
	as
	(
		select 1
		union all
		select Num + 1
		from CTE
		where Num < 10000
	)
	insert into dbo.BlobData(ID, CharField)
		select Num, 'placeholder'
		from CTE
	option (maxrecursion 0)
commit
go

if exists(
	select * 
	from sys.views v join sys.schemas s on v.schema_id = s.schema_id
	where s.name = 'dbo' and v.name = 'vMainAndBlobData'
)
	drop view dbo.vMainAndBlobData
go

-- need to use outer join
create view dbo.vMainAndBlobData(ID, IntField, CharField)
as
	select md.ID, md.IntField, bd.CharField 
	from 
		dbo.MainData md left outer join dbo.BlobData bd
			on md.ID = bd.ID
go
		
/* When you need blob field.. */
select ID, IntField 
from dbo.vMainAndBlobData
where ID = 5
go

/* Select does not require CharField - Original data */
dbcc dropcleanbuffers
dbcc freeproccache
go

set statistics io on
go

declare 
	@StartTime datetime = getDate()

select 	COUNT(*) from dbo.LargeRow where IntField % 2 = 0
select datediff(millisecond,@StartTime,getDate())
go

/* vMainAndBlobData */
dbcc dropcleanbuffers
dbcc freeproccache
go

set statistics io on
go

declare 
	@StartTime datetime = getDate()

select 	COUNT(*) from dbo.vMainAndBlobData where IntField % 2 = 0
select datediff(millisecond,@StartTime,getDate())
go
