/******************************************************
Tampa IASA (Tampa, FL. 2011-02-24)

Dmitri V. Korotkevitch "Database Design for non-database architects"

( dmitri@aboutsqlserver.com )

This demo shows how usage of non-clustered index
depends on the selectivity of the index.

IndexDemo table created and populated in "Init Db.sql" script

Enable "Include Actual Execution Plan" option
******************************************************/
use IASA	
go


if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'IndexDemo'
)
	drop table dbo.IndexDemo
go

create table dbo.IndexDemo
(
	Id int not null identity(1,1),
	Name char(20) not null,
	Placeholder char(379) not null,

	constraint PK_IndexDemo 
	primary key clustered(id)
)
go

declare
	@First int = 0
	,@Second int
	,@Third int
	,@C char(3)
	,@I int 	
begin tran
	while @First < 26
	begin
		select @Second = 0
		while @Second < 26
		begin
			select @Third = 0
			while @Third < 26
			begin
				select @C = CHAR(65 + @First) + CHAR(65 + @Second) + CHAR(65 + @Third)
				insert into dbo.IndexDemo(Name,Placeholder) 
				values(@C,'a'), (@C,'a'), (@C,'a'), (@C,'a'), (@C,'a')
				select @Third = @Third + 1
			end
			select @Second = @Second + 1
		end
		select @First = @First + 1
	end
commit				 

create index IDX_IndexDemo_Name on dbo.IndexDemo(Name)
go


select COUNT(*) from dbo.IndexDemo
go

select page_count, * from sys.dm_db_index_physical_stats(DB_ID(),OBJECT_ID(N'IndexDemo'),null, null, 'DETAILED')
go

-- Clustered index = 4,626 pages
-- Non Clustered index = 296 pages

set statistics io on
go

-- 5 rows. SQL uses non-clustered index
select * from dbo.IndexDemo where Name like 'ABC%'
go

-- 130 rows. SQL uses non-clustered index
select * from dbo.IndexDemo where Name like 'AB%'
go

-- 3380 rows - SQL uses clustered index scan 4643 reads
select * from dbo.IndexDemo where Name like 'A%'
go

-- 10375 reads - more than 3 times more expensive than with clustered index scan
select * from dbo.IndexDemo with (index = IDX_IndexDemo_Name)  where Name like 'A%'
go

